import functools
import os
import re
import subprocess

import click


def task(msg_before, msg_after=None):
    def outer_wrapper(func):
        @functools.wraps(func)
        def inner_wrapper(*args, **kwargs):
            click.echo(msg_before, nl=not msg_after)
            func(*args, **kwargs)
            if msg_after:
               click.echo(msg_after)
        return inner_wrapper
    return outer_wrapper


def substitute(path, pattern, replacement):
    with open(path, 'r') as f:
        content = f.read()

    content = re.sub(pattern, replacement, content)

    with open(path, 'w') as f:
        f.write(content)


def replace(path, subs):
    with open(path, 'r') as f:
        content = f.read()

    if content:
        for key, value in subs.items():
            content = content.replace(key, value)
        with open(path, 'w') as f:
            f.write(content)


def shell(cmd, root=None):
    """ Silently perform a shell command.

    Parameters
    ----------
    cmd: str
        The command to perform.

    root: path or str
        The directory where to perform the command. Default: current directory.
    """
    if root:
        cmd = 'cd ' + root + '; ' + cmd
    subprocess.run(cmd, shell=True, stdout=subprocess.DEVNULL)


def render(path, template=None, subs=None):
    """ Write a file based on a file or string template.

    Parameters
    ----------
    path: str or Pathlib path
        The path of the file to write.

    template: Path or str
        The file path or the string used as template.

    subs: dict
        The substitutions to perform on the template.
    """
    if template:
        if os.path.exists(template):
            with open(template, 'r') as f:
                content = f.read()
        else:
            content = template
    else:
        content = ''

    if subs:
        for key, value in subs.items():
            content = content.replace(key, str(value))

    with open(path, 'w') as f:
        f.write(content)




def help_tests():
    help_text = """

How to run tests
****************

   python -m unittest discover tests
    
"""

    print(help_text)


def help_install():
    help_text = """

How to install project for development
**************************************

In order to install your own project for development, install it in
development mode (a.k.a editable mode). From the project root directory,
submit:

    pip install -e .

or

    python setup.py develop
    
"""
    print(help_text)


import os
import pathlib
import datetime

import click

from manati.utils import task, replace, shell, render
from manati.add import add_license


def create_project(name, no_git, no_install, author, description, license):
    """ Create a Python project structure in the current working directory.

    Parameters
    ----------
    name: str
        Name of the project. Also the main package name.

    no_git: bool
        If true, do not create local git repository.

    no_install: bool
        If true, do not pip-install in editable mode.

    author: str
        Author of the project.

    description:
        Short description of the project. Used for README and setup.py.
    """

    path = pathlib.Path.cwd() / name
    if os.path.exists(path):
        raise Exception('ERROR: Path already exists.')

    templates = pathlib.Path(__file__).parent / 'templates'
    subs = {
        'PROJECT_NAME': name,
        'YEAR': datetime.date.today().year,
        'VERSION': '0.0.1',
        'MODULE_NAME': 'main',
        'AUTHOR': author,
        'DESCRIPTION': description
    }

    create_project_structure(name, path, subs, templates)

    if license != 'None':
        add_license(path, license)

    create_docs(path, name, author)
    build_documentation(name)

    if not no_git:
        create_git_repo(name)

    if not no_install:
        pip_install_project(name)

    click.echo('Happy coding!')


@task('Create project structure...', ' OK')
def create_project_structure(name, path, subs, templates):
    os.makedirs(path)
    os.makedirs(path / name)
    render(path / 'README.md', templates / 'README.md', subs)
    render(path / 'setup.py', templates / 'setup.py', subs)
    render(path / name / 'main.py', templates / 'source.py', subs)
    render(path / name / '__init__.py')
    os.makedirs(path / 'tests')
    render(path / 'tests' / 'test_main.py', templates / 'test.py', subs)
    render(path / '.gitignore', templates / '.gitignore')


@task('Pip-installing as editable version...', 'OK')
def pip_install_project(name):
    shell('pip install -e .', root=name)


@task('Create local git repository...', ' OK')
def create_git_repo(name):
    shell('git init', root=name)


@task('Creating project documentation...', ' OK')
def create_docs(path, name, author):
    """ Create a project documentation.

    Parameters
    ----------
    path: str or Pathlib path
        Path of the project root. Folder 'docs' will be created as subfolder.

    name: str
        Name of the project

    author:
        Author of the project
    """

    os.makedirs(path / 'docs')
    render(path / 'docs' / 'requirements.txt', 'sphinx\nsphinx_rtd_theme\n')

    shell('pip install -r requirements.txt', root=str(path) + '/docs')

    cmd = 'sphinx-quickstart -p %s -a "%s" -v 0.0.1 --no-sep -l en -r 0.0.1 docs' % (name,
                                                                    author,
                                                                    )
    shell(cmd, str(path))

    replace(path / 'docs' / 'conf.py', {'alabaster': 'sphinx_rtd_theme'})


@task('Build documentation...', ' OK')
def build_documentation(name):
    shell('make html', root=name + '/docs')



import pathlib
import shutil
import os

import click

from manati.create import create_project, create_docs
from manati.add import add_package, add_license
from manati.apropos import help_tests, help_install
from manati.validators import validate_project_name


@click.group('manati')
def cli(*args, **kwargs):
    """Manati - a command line interface (CLI) for managing Python projects."""
    pass


@cli.command('create')
@click.option('-n', '--name', 'name', required=True, callback=validate_project_name,
              prompt='Project name', help='Name of the project, same as the main package.')
@click.option('-G', '--no-git', 'no_git', is_flag=True, default=False,
              help='Do not create git repository')
@click.option('-I', '--no-install', 'no_install', is_flag=True, default=False,
              help='Do not pip-install in editable mode')
@click.option('-a', '--author', 'author', default='AUTHOR', prompt='Author')
@click.option('-d', '--description', 'description', default='', prompt='(Short) description')
@click.option('-l', '--license', 'license', type=click.Choice([
'MIT', 'GPLv3', 'Apache', 'None'
], case_sensitive=False), prompt='License', default='None')
def create_project_command(name, no_git, no_install, author, description, license):
    """Create a standard Python project structure.

    By default, the project is also pip-installed for development
    in editable mode, and a local git repository is also created.
    """
    try:
        create_project(name, no_git, no_install, author, description, license)
    except Exception as e:
        click.echo(e)


@cli.group('add')
def add(*args, **kwargs):
    """Adds something to the current project."""
    pass


@add.command('docs')
def add_docs_command():
    """Add a docs folder with Sphinx documentation to the current directory."""
    cwd = pathlib.Path.cwd()
    create_docs(cwd, 'PROJECT_NAME', 'AUTHOR')


@add.command('package')
@click.argument('package_name')
def add_package_command(package_name):
    """Add a package to the current directory.

    PACKAGE_NAME must be the fully qualified package name, e.g.

         manati add package myproject.mypackage
    """
    click.echo('Create package...', nl=False)
    add_package(package_name)
    click.echo('Done.')


@add.command('license')
@click.option('-n', '--name', 'name', type=click.Choice(['MIT', 'GPLv3', 'Apache', 'None'], case_sensitive=False),
              required=True, prompt='License')
def add_license_command(name):
    """Add a license to the current project."""
    add_license(pathlib.Path.cwd(), name)


@add.command('gitignore')
def add_gitignore_command():
    """Add a default .gitignore file to the current directory."""
    cwd = pathlib.Path.cwd()
    target = cwd / '.gitignore'
    source = pathlib.Path(__file__).parent / 'templates' / '.gitignore'

    if not os.path.exists(target):
        shutil.copyfile(source, target)
    else:
        if click.confirm('.gitignore file already exists in current directory. Overwrite?'):
            shutil.copyfile(source, target)


@add.command('setup.py')
def add_setup_py_command():
    """Add a setup.py file to the current directory"""
    cwd = pathlib.Path.cwd()
    target = cwd / 'setup.py'
    source = pathlib.Path(__file__).parent / 'templates' / 'setup.py'

    if not os.path.exists(target):
        shutil.copyfile(source, target)
    else:
        if click.confirm('setup.py file already exists in current directory. Overwrite?'):
            shutil.copyfile(source, target)


@cli.group('apropos')
def apropos(*args, **kwargs):
    """Print reminders on various topics."""
    pass


@apropos.command('tests')
def apropos_tests_command():
    """Help on how to run tests."""
    help_tests()


@apropos.command('install')
def apropos_install_command():
    """Help on how to install in development mode."""
    help_install()


if __name__ == '__main__':
    cli()


import pathlib
import os
import shutil

import click

from manati.utils import task, substitute


@task('Create package...', ' OK')
def add_package(name):
    """ Add a package structure in the current working directory

    Parameters
    ----------
    name: str
        Name of the package including parent packages.
    """
    components = name.split('.')

    path = pathlib.Path('.')

    for comp in components:
        path = path / comp
        if not os.path.exists(path):
            os.mkdir(path)
        if not os.path.exists(path / '__init__.py'):
            with open(path / '__init__.py', 'w') as f:
                f.write('')


def add_license(path, license):

    if os.path.exists(path / 'LICENSE'):
        raise click.BadParameter('LICENSE file already existing.')

    templates_dir = pathlib.Path(__file__).parent / 'templates' / 'licenses'

    files = {'MIT': templates_dir / 'MIT',
             'GPLv3': templates_dir / 'GPLv3',
             'Apache': templates_dir / 'Apache'
             }

    shutil.copyfile(files[license], path / 'LICENSE')

    setup_py = path / 'setup.py'
    if not os.path.exists(setup_py):
        click.echo('Warning: cannot find setup.py to set license type.')
        return
    substitute(setup_py, "license='None'", "license='%s'" % license)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import exists
from setuptools import setup, find_packages

author = 'maroba'
email = 'matthias.r.baer@googlemail.com'
description = 'A Python CLI for managing Python projects.'
name = 'manati'
year = '2021'
url = 'https://github.com/maroba/manati'
version = '0.1.0'

setup(
    name=name,
    author=author,
    author_email=email,
    url=url,
    version=version,
    packages=find_packages(),
    package_dir={name: name},
    include_package_data=True,
    entry_points='''
        [console_scripts]
        manati=manati.manati:cli
    ''',
    license="GPL v3",
    description=description,
    long_description=open('README.md').read() if exists('README.md') else '',
    long_description_content_type="text/markdown",
    install_requires=['Click',
                      'sphinx'
                      ],
    # not to be confused with definitions in pyproject.toml [build-system]
    python_requires=">=3.6",
    keywords=["CLI"],
    classifiers=['Operating System :: OS Independent',
                 'Programming Language :: Python :: 3',
                 ],
    platforms=['ALL'],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import exists
from setuptools import setup, find_packages

author = 'AUTHOR'
email = 'EMAIL'
description = 'DESCRIPTION'
name = 'PROJECT_NAME'
year = 'YEAR'
url = 'URL'
version = 'VERSION'

setup(
    name=name,
    author=author,
    author_email=email,
    url=url,
    version=version,
    packages=find_packages(),
    package_dir={name: name},
    include_package_data=True,    
    license='None',
    description=description,
    long_description=open('README.md').read() if exists('README.md') else '',
    install_requires=['sphinx',
                      ],    
    python_requires=">=3.6",    
    classifiers=['Operating System :: OS Independent',
                 'Programming Language :: Python :: 3',
                 ],
    platforms=['ALL'],
)
